<?php
namespace Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\Log\Exception;


abstract class Crud extends Simple {

    /** @var Manager The Entity Manager used to interact with Database. */
    protected $dataSourceManager;
    /** @var Manager The Entity Manager used to interact with Database. */
    protected $dataSourceName;

    /**
     * Constructor...
     *
     * @param Manager $dataSourceManager     The DataSourceManager implementation used by your product.
     */
    public function __construct(Manager $dataSourceManager) {
        $this->dataSourceManager = $dataSourceManager;
    }

    /**
     * Action used for insert new rows and update the old ones.
     *
     * @param DTO\Request\DataSet $request  A DTO\Request\DataSet with all rows to be persisted..
     * @param DTO\Response        $response A DTO\Response, mostly of cases will only add messages.
     *
     * @return void
     */

	public function save(Request $request, Response $response, $showMessage = true) {
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        $persistedRows = $this->dataSourceManager->persist($dataSet);
        $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        if($showMessage) {
            $nrPersistedRows = count($persistedRows);
            $response->addMessage(new Message("{$nrPersistedRows} rows persisted with success"));
        }
    }

    /**
     * Action used for find rows that match with given FilterCriteria
     *
     * @param DTO\Request\Filter $request A DTO\Request with FilterCriteria to match the rows in DataSourceManager.
     * @param Response $response
     * @internal param Response $response A DTO\Response to be added with DataSets of matched rows.
     *
     * @return void
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }
    /**
     * Action used for find rows that match with given FilterCriteria
     *
     * @param DTO\Request\Filter $request A DTO\Request with FilterCriteria to match the rows in DataSourceManager.
     * @param Response $response
     * @param $dataSourceName
     * @internal param Response $response A DTO\Response to be added with DataSets of matched rows.
     *
     * @return void
     */
    public function findSetNameDataSet(DTO\Request\Filter $request, DTO\Response $response,$dataSourceName = null) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        if($dataSourceName){
            $dataSet->setDataSourceName($dataSourceName);
        }
        $response->addDataSet($dataSet);
    }

    /**
     * Action used for delete some rows.
     *
     * @param DTO\Request\DataSet $request  A DTO\Request with DataSet of all rows to be deleted..
     * @param DTO\Response        $response A DTO\Response, mostly of cases will only add messages.
     *
     * @return void
     */
    public function delete(Request $request, Response $response, $showMessage = true) {
        $request = $this->buildDataSetIfRow($request);
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        $deletedRows = $this->dataSourceManager->delete($dataSet);
        $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        if($showMessage) {
            $nrDeletedRows = count($deletedRows);
            $response->addMessage(new Message("{$nrDeletedRows} row(s) deleted."));
        }
    }

    protected function buildDataSetIfRow(Request $request) {
        if($request instanceof Request\Row) {
            $dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
            $request = new Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        }
        return $request;
    }
}